@@ -1,4 +1,5 @@
 import time
+from datetime import datetime, timedelta
 from typing import TYPE_CHECKING, Any, Dict, Generator, List, Sequence, Set, Union
 
 from ..aliases import PathOrStr
@@ -187,7 +188,7 @@ class ExperimentClient(ServiceClient):
             ).json()
         )
 
-    def tasks(self, experiment: Union[str, Experiment]) -> List[Task]:
+    def tasks(self, experiment: Union[str, Experiment]) -> Tasks:
         """
         List the tasks in an experiment.
 
@@ -196,9 +197,14 @@ class ExperimentClient(ServiceClient):
         :raises ExperimentNotFound: If the experiment can't be found.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
         :raises HTTPError: Any other HTTP exception that can occur.
+
+        :examples:
+
+        >>> task = beaker.experiment.tasks(hello_world_experiment_name)["main"]
+
         """
         experiment_id = self.resolve_experiment(experiment).id
-        return [
+        tasks = [
             Task.from_json(d)
             for d in self.request(
                 f"experiments/{self.url_quote(experiment_id)}/tasks",
@@ -208,12 +214,14 @@ class ExperimentClient(ServiceClient):
                 },
             ).json()
         ]
+        return Tasks(tasks)
 
     def logs(
         self,
         experiment: Union[str, Experiment],
         task: Optional[Union[str, Task]] = None,
         quiet: bool = False,
+        since: Optional[Union[str, "datetime", "timedelta"]] = None,
     ) -> Generator[bytes, None, None]:
         """
         Download the logs for an experiment.
@@ -232,6 +240,10 @@ class ExperimentClient(ServiceClient):
         :param task: The task ID, name, or object of a specific task from the Beaker experiment
             to fetch logs for. Required if there are multiple tasks in the experiment.
         :param quiet: If ``True``, progress won't be displayed.
+        :param since: Only show logs since a particular time. Could be a :class:`~datetime.datetime` object
+            (naive datetimes will be treated as UTC), a timestamp string in the form of RFC 3339
+            (e.g. "2013-01-02T13:23:37Z"), or a relative time
+            (e.g. a :class:`~datetime.timedelta` or a string like "42m").
 
         :raises ValueError: The experiment has no tasks or jobs, or the experiment has multiple tasks but
             ``task`` is not specified.
@@ -250,7 +262,7 @@ class ExperimentClient(ServiceClient):
                     f"Experiment {exp.id} has no jobs for task "
                     f"'{task if isinstance(task, str) else task.display_name}'"
                 )
-        return self.beaker.job.logs(job.id, quiet=quiet)
+        return self.beaker.job.logs(job.id, quiet=quiet, since=since)
 
     def metrics(
         self, experiment: Union[str, Experiment], task: Optional[Union[str, Task]] = None
@@ -330,6 +342,9 @@ class ExperimentClient(ServiceClient):
         .. seealso::
             :meth:`as_completed()`
 
+        .. seealso::
+            :meth:`follow()`
+
         .. seealso::
             :meth:`Beaker.job.wait_for() <JobClient.wait_for>`
 
@@ -341,7 +356,7 @@ class ExperimentClient(ServiceClient):
             :class:`~beaker.exceptions.JobFailedError` will be raised for non-zero exit codes.
 
         :raises ExperimentNotFound: If any experiment can't be found.
-        :raises TimeoutError: If the ``timeout`` expires.
+        :raises JobTimeoutError: If the ``timeout`` expires.
         :raises DuplicateExperimentError: If the same experiment is given as an argument more than once.
         :raises JobFailedError: If ``strict=True`` and any job finishes with a non-zero exit code.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
@@ -384,6 +399,9 @@ class ExperimentClient(ServiceClient):
         .. seealso::
             :meth:`wait_for()`
 
+        .. seealso::
+            :meth:`follow()`
+
         .. seealso::
             :meth:`Beaker.job.as_completed() <JobClient.as_completed>`
 
@@ -395,7 +413,7 @@ class ExperimentClient(ServiceClient):
             :class:`~beaker.exceptions.JobFailedError` will be raised for non-zero exit codes.
 
         :raises ExperimentNotFound: If any experiment can't be found.
-        :raises TimeoutError: If the ``timeout`` expires.
+        :raises JobTimeoutError: If the ``timeout`` expires.
         :raises DuplicateExperimentError: If the same experiment is given as an argument more than once.
         :raises JobFailedError: If ``strict=True`` and any job finishes with a non-zero exit code.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
@@ -406,7 +424,7 @@ class ExperimentClient(ServiceClient):
 
         from ..progress import get_exps_and_jobs_progress
 
-        start = time.time()
+        start = time.monotonic()
         # Initialize progress trackers.
         live_display, experiments_progress, jobs_progress = get_exps_and_jobs_progress(quiet)
         # Keeps track of the IDs of each job for each task in each experiment.
@@ -480,9 +498,9 @@ class ExperimentClient(ServiceClient):
                         yield complete_experiment(exp_id)
 
                 # Check for timeout.
-                elapsed = time.time() - start
+                elapsed = time.monotonic() - start
                 if timeout is not None and elapsed >= timeout:
-                    raise TimeoutError
+                    raise JobTimeoutError
 
                 if incomplete_jobs:
                     # Wait for current stack of incomplete jobs to finalize.
@@ -499,11 +517,8 @@ class ExperimentClient(ServiceClient):
                         exp_id = job.execution.experiment
 
                         # Ensure job was successful if `strict==True`.
-                        if strict and job.status.exit_code != 0:
-                            raise JobFailedError(
-                                f"Job '{job.id}' from experiment '{exp_id}' failed "
-                                f"with exit code '{job.status.exit_code}'"
-                            )
+                        if strict:
+                            job.check()
 
                         # Update progress display.
                         experiments_progress.advance(exp_to_progress_task[exp_id])
@@ -534,6 +549,78 @@ class ExperimentClient(ServiceClient):
                         assert latest_job is not None
                         task_to_job[task.id] = latest_job.id
 
+    def follow(
+        self,
+        experiment: Union[str, Experiment],
+        task: Optional[Union[str, Task]] = None,
+        timeout: Optional[float] = None,
+        strict: bool = False,
+    ) -> Generator[bytes, None, Experiment]:
+        """
+        Follow an experiment live, creating a generator that produces log lines
+        (as bytes) from the task's job as they become available.
+        The return value of the generator is the final
+        :class:`~beaker.data_model.experiment.Experiment` object.
+
+        .. seealso::
+            :meth:`logs()`
+
+        .. seealso::
+            :meth:`wait_for()`
+
+        .. seealso::
+            :meth:`as_completed()`
+
+        .. seealso::
+            :meth:`Beaker.job.follow() <JobClient.follow>`
+
+        :param experiment: Experiment ID, name, or object.
+        :param task: The task ID, name, or object of a specific task from the Beaker experiment
+            to follow. Required if there are multiple tasks in the experiment.
+        :param timeout: Maximum amount of time to wait for (in seconds).
+        :param strict: If ``True``, the exit code of the job will be checked, and a
+            :class:`~beaker.exceptions.JobFailedError` will be raised for non-zero exit codes.
+
+        :raises ExperimentNotFound: If any experiment can't be found.
+        :raises ValueError: The experiment has no tasks or jobs, or the experiment has multiple tasks but
+            ``task`` is not specified.
+        :raises TaskNotFound: If the given task doesn't exist.
+        :raises JobTimeoutError: If the ``timeout`` expires.
+        :raises JobFailedError: If ``strict=True`` and the task's job finishes with a non-zero exit code.
+        :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
+        :raises HTTPError: Any other HTTP exception that can occur.
+
+        :examples:
+
+        >>> for line in beaker.experiment.follow(hello_world_experiment_name):
+        ...     # Every log line from Beaker starts with an RFC 3339 UTC timestamp
+        ...     # (e.g. '2021-12-07T19:30:24.637600011Z'). If we don't want to print
+        ...     # the timestamps we can split them off like this:
+        ...     line = line[line.find(b"Z ")+2:]
+        ...     print(line.decode(errors="ignore"), end="")
+        <BLANKLINE>
+        Hello from Docker!
+        This message shows that your installation appears to be working correctly.
+        <BLANKLINE>
+        ...
+        """
+        if timeout is not None and timeout <= 0:
+            raise ValueError("'timeout' must be a positive number")
+
+        start = time.monotonic()
+        job: Optional[Job] = None
+        while job is None:
+            job = self.latest_job(experiment, task=task)
+            if timeout is not None and time.monotonic() - start >= timeout:
+                raise JobTimeoutError(
+                    "Job for task failed to initialize within '{timeout}' seconds"
+                )
+            time.sleep(2.0)
+
+        assert job is not None  # for mypy
+        yield from self.beaker.job.follow(job, strict=strict)
+        return self.get(experiment.id if isinstance(experiment, Experiment) else experiment)
+
     def url(
         self, experiment: Union[str, Experiment], task: Optional[Union[str, Task]] = None
     ) -> str:
@@ -586,7 +673,7 @@ class ExperimentClient(ServiceClient):
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
         :raises HTTPError: Any other HTTP exception that can occur.
         """
-        tasks = self.tasks(experiment)
+        tasks = list(self.tasks(experiment))
         exp_id = experiment if isinstance(experiment, str) else experiment.id
         if not tasks:
             raise ValueError(f"Experiment '{exp_id}' has no tasks")
